/* #333A73 #00A9FF #89CFF3 #A0E9FF #CDF5FD*/

/*
.mantine-MultiSelect-value {
  background-color: #333A73;
  color: white;
  font-family: 'Roboto';
}

.mantine-MultiSelect-defaultValueRemove {
  color: white;
}
*/
.mantine-Grid-col {
  padding-top: 0;
}

.mantine-Slider-track {
  background-color: white;
  border-color: #000000;
  border: 100px;
}

body {
  font-family: 'Roboto';
}

h1 {
  font-family: 'Roboto';
}

/*
.mantine-Slider-bar {
  background-image: linear-gradient(45deg, #89CFF3 0%, #333A73  51%, #00A9FF  100%);
}
.mantine-Slider-thumb {
    border-color: #00A9FF;
    height: 20px;
    width: 20px;
    background-color: white;
}
*/

/*
.mantine-Button-root {
  margin: 10px;
  padding: 10px 18px;
  text-align: center;
  transition: 0.5s;
  background-size: 200% auto;
  color: white;
  border-radius: 10px;
  display: block;
  border: 0px;
  font-weight: 700;
  background-image: linear-gradient(90deg, #89CFF3 0%, #333A73  51%, #00A9FF  100%);
  cursor: pointer;
  user-select: none;
  -webkit-user-select: none;
  touch-action: manipulation;
}

.mantine-Button-root:hover {
  background-position: right center;
  color: #fff;
  text-decoration: none;
}

.mantine-Button-root:active {
  transform: scale(0.95);
}

.mantine-Button-root:disabled {
  background-color: #333A73;
  color:#BABABA ;
}
*/
